"""
FlowMason Lab - Component Development Workspace.

This package contains:
- Core nodes (generator, critic, improver, synthesizer, selector)
- Core operators (http_request, json_transform, filter, loop, etc.)
- Custom/specialized components

All components are packaged as .fmpkg files and deployed to FlowMason instances.
"""

__version__ = "1.0.0"
