"""
FlowMason Studio Setup Module.

Contains utilities for initializing new FlowMason instances.
"""

from flowmason_studio.setup.core_packages import CorePackageInstaller

__all__ = ["CorePackageInstaller"]
